/*
 * Kansi.java
 *
 * Created on 2008/01/17, 18:54
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.kyuureki;
import static to.tetramorph.kyuureki.KansiConst.*;
/**
 * 年月日時それぞれの、干支と九星を表現する。<br>
 * KansiKyuuseiAlmanacから、グレゴリオ暦に対する干支や九星の情報は、このクラスで
 * 返却される。
 * 干支や九星はフィールドから直接アクセスする。
 * ただしメソッドはフィールドに矛盾のあるデータが書きこまれていた場合、
 * Exceptionを出す場合がある。
 * @author 大澤義鷹
 */
public class Kansi {
    public static final int YEAR = 0;
    public static final int MONTH = 1;
    public static final int DAY = 2;
    public static final int HOUR = 3;
    public int 年干 = -1;
    public int 月干 = -1;
    public int 日干 = -1;
    public int 時干 = -1;
    public int 年支 = -1;
    public int 月支 = -1;
    public int 日支 = -1;
    public int 時支 = -1;
    public int 年家九星 = -1;
    public int 月家九星 = -1;
    public int 日家九星 = -1;
    public int 時家九星 = -1;
    public int 年三元 = -1;
    public int 月三元 = -1;
    //public int 日三元 = -1; //まだ未対応
    public int 時三元 = -1;
    /**
     * このフラグがtrueのとき節入判定暦の範囲外で、節入時刻の判定が計算結果に
     * 反映されていないことを表す。初期値はtrue。
     */
    public boolean isSetuiriUnknown = true;
    /**
     * 日家九星の陰遁/陽遁を表すフラグで、1なら陽遁、-1なら陰遁。
     * 初期値は0で未定義を表す。
     */
    public int 遁行 = 0;
    /**
     * 日家九星でその九星が置閏(ちじゅん)期間に該当するときは1。違うときは0。
     * 初期値は-1で未定義を表す。
     */
    public int 置閏 = -1;

    /**  Kansi オブジェクトを作成する */
    public Kansi() {
    }
    /**
     * 干支名を返す。干や支のフィールドに範囲外の値が入っている場合はnullを返す。
     * 初期値はそれぞれ-1なので、値をセットせずに呼び出せばnullが返る。
     * @param field YEAR,MONTH,DAY,HOURのいずれかを指定。
     * @exception IllegalArgumentException fieldにYEAR,MONTH,DAY,HOUR以外の値が
     * 指定さた場合と、干支の組合せでありえないものがセットされている場合。
     */
    public String get干支(int field) {
        if ( ! ( field >= YEAR && field <= HOUR ) ) {
            throw new IllegalArgumentException("引数異常");
        }
        int 干 = new int[] { 年干,月干,日干,時干 }[ field ];
        int 支  = new int[] { 年支,月支,日支,時支 }[ field ];
        if ( 干 < 0 || 干 >= 10 || 支<0 || 支 >= 12 ) {
            return null;
        }
        int num = 六十干支番号表 [ 干 ][ 支 ];
        if ( num < 0 ) {
            throw new IllegalArgumentException("ありえない干支の組合せ");
        }
        return 六十干支[num];
    }
    
    /**
     * 九星名を返す。もし1-9以外の値が設定されている場合はnullを返す。
     * @param field YEAR,MONTH,DAY,HOURのいずれかを指定。
     */
    public String get九星(int field) {
        if ( ! ( field >= YEAR && field <= HOUR ) ) {
            throw new IllegalArgumentException("引数異常");
        }
        int k = new int [] { 年家九星,月家九星,日家九星,時家九星 }[field];
        if( k >= 1 && k<= 9) {
             return 九星[k];
        }
        return null;
    }
    
    /**
     * 九星名の略称(一白、二黒、等)を返す。
     * @param field YEAR,MONTH,DAY,HOURのいずれかを指定。
     */
    public String get九星略称(int field) {
        if ( ! ( field >= YEAR && field <= HOUR ) ) {
            throw new IllegalArgumentException("引数異常");
        }
        int k = new int [] { 年家九星,月家九星,日家九星,時家九星 }[field];
        if( k >= 1 && k<= 9) {
             return 九星略称[k];
        }
        return null;
    }
    
    /**
     * 日家九星の陽遁/陰遁を文字列で返す。このメソッドの返す値は、遁行フィールド
     * の値が1なら陽遁-1なら陰遁、それ以外の値はnullを返す。
     */
    public String get遁行() {
        if ( 遁行 == 1 ) return "陽遁";
        else if ( 遁行 == -1 ) return "陰遁";
        return null;
    }

    private static final String [] title = { "(年)","(月)","(日)","(時)" };
    /**
     * このオブジェクトを表現する文字列を返す。
     */
    public String toString() {
        StringBuilder sb = new StringBuilder(80);
        for(int i=YEAR; i<=HOUR; i++) {
            sb.append(get干支(i));
            sb.append(" ");
            sb.append(get九星略称(i));
            sb.append(title[i]);
            sb.append(",");
        }
        sb.append("遁行=");
        sb.append(遁行);
        sb.append(",");
        sb.append("年三元=");
        sb.append(年三元);
        sb.append(",");
        sb.append("月三元=");
        sb.append(月三元);
        sb.append(",");
//        sb.append("日三元=");
//        sb.append(日三元);
//        sb.append(",");
        sb.append("時三元=");
        sb.append(時三元);
        return sb.toString();
    }

}
