/*
 * KansiKyuuseiAlmanac.java
 *
 * Created on 2008/01/21, 0:49
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.kyuureki;

import java.util.Calendar;
import to.tetramorph.time.JDay;

/**
 * グレゴリオ暦の年月日時刻から干支と九星を求める暦。<br>
 * 節入時刻のデータはグレゴリオ暦開始の次の年からしかもっていないので、
 * 節入を考慮した計算ができるのは1583年1月から2100年まで。<br>
 * 節入を考慮せず日の干支や九星を求めることは、AD1年からAD2260年まで。
 * 節入計算可能な範囲を越えた場合は、かならず節入したものとして計算する。
 * その際、計算結果のKansi#isSetuiriUnknownのフラグはtrueとなる。<br>
 * <pre>
 * 使い方
 *    Calendar cal = cal.getInstance();
 *    cal.set(year,month-1,day,hour,minute,second);
 *    cal.set(Calendar.MILLISECOND,0);
 *    Kansi kansi = KansiKyuuseiAlmanac.getValue(cal);
 * kansiにグレゴリオ暦に対する年月日時の干支と九星が求まる。
 * </pre>
 * 
 * @author 大澤義鷹
 */
public class KansiKyuuseiAlmanac {
    
    private KansiKyuuseiAlmanac() {
    }
    
    /**
     * get()の端数を切り捨てて、UTCにおける午前0時の値にして返す。
     */
    private static int ijday(int year,int month,int day) {
        return (int)JDay.get( year, month, day);
    }
    
    /**
     * 指定されたユリウス日から日家九星を求めresultに書きこむ。
     */
    private static void get日家九星(int jday,Kansi result) {
        int jd = 2421190; //get(1916,11,23)-0.5;
        int jq = (int)jday;
        int c = 0;
        if (jq >= 2484010) {        // AD2088年より未来のとき
            jd = 2484010;
            c = 1;
        } else {
            for ( ; jq < jd; c++ ) { //AD1916-11-23より過去のとき
                jd -= (63000-180);
            }
        }
        int totalOffset = jq - jd;
        int c63k = c & 1;
        int c4200 = (int)(totalOffset / 4200);
        int u = (int)(totalOffset % 4200);

        int 遁行 = 0, 置閏 = 0, 九星 = 0;
        if ( (( c4200 & 1)) == 0) {           //0なら冬至付近から始まる場合
            if ( u < 60 ) {   //置閏期間
                置閏 =  1;
                if ( u < 30 ) {
                    九星 = 9 - (u % 9);          //pmod(8-u,9) + 1;
                    遁行 = -1;
                } else {
                    九星 = (u+3) % 9 + 1;
                    遁行 =  1;
                }
            } else {          //通常
                int c180 = (int)((u-60)/180);
                if ( (c180 & 1) == 0 ) {
                    九星 = (u+3) % 9 + 1;
                    遁行 =  1;
                } else {
                    九星 = 9 - ((u+3) % 9);     //pmod( 5-u, 9) + 1;
                    遁行 = -1;
                }
            }
        } else {                                     //夏至付近から始まる場合
            if ( u < 60 ) {   //置閏期間
                置閏 =  1;
                if ( u < 30 ) {
                    九星 = u % 9 + 1;
                    遁行 =  1;
                } else {
                    九星 = 9 - ((u+3) % 9);     //pmod(5-u, 9) + 1;
                    遁行 = -1;
                }
            } else {          //通常
                int c180 = (int)((u-60)/ 180);
                if ( (c180 & 1) == 0 ) {
                    九星 = 9 - ((u+3) % 9);     //pmod(5-u, 9) + 1;
                    遁行 = -1;
                } else {
                    九星 = (u+3) % 9 + 1;
                    遁行 =  1;
                }
            }
        }
        result.日家九星 = 九星;
        result.遁行 = 遁行;
        result.置閏 = 置閏;
    }
    /**
     * カレンダーで指定された年月日時の干支と九星を求める。
     * @param cal カレンダー
     * @param result 結果が書きこまれる
     * @param isLenient 節入り判定を無視した結果を求める場合はtrueを指定する。
     * 無視するのはカレンダー用途。鑑定のときはは節入りを考慮する必要がある。
     */
    public static void getValue( Calendar cal, Kansi result, boolean isLenient ) {
        SetuiriAlmanac setuiri = SetuiriAlmanac.getInstance();
        int year   = cal.get( Calendar.YEAR );
        int month  = cal.get( Calendar.MONTH ) + 1;
        int day    = cal.get( Calendar.DAY_OF_MONTH );
        int hour   = cal.get( Calendar.HOUR_OF_DAY );
        boolean isSetuiri = false;
        boolean isSetuiriUnknown = false;
        try {
            isSetuiri = setuiri.isSetuiri(cal);
        } catch (Exception e) {
            isSetuiriUnknown = true; //節入時刻が求められない場合
            isSetuiri = true;        //つねに節入したものとみなす
        }
        if ( isLenient ) isSetuiri = true;
        
        result.isSetuiriUnknown = isSetuiriUnknown;
        
        //年の干支を求める
        int yk = year;
        if ( ! isSetuiri ) {           //2月節入り前なら前年とみなす
            yk = (month <= 2) ? yk - 1 : yk;
        }
        result.年干   = ( yk - 4 ) % 10;
        result.年支   = ( yk - 4 ) % 12;
        
        //月の干支を求める
        int m = month;
        int y = year;
        if ( ! isSetuiri ) {           //節入り前なら前月とする。
            if( (m - 1) == 0 ) {
                y--;                   //1月節入前なら前年の12月とする
                m = 12;
            } else {
                m--;                  //節入前なので前月とする
            }
        }
        result.月干   = ( y * 12 + m + 2 ) % 10;
        result.月支   = ( y * 12 + m ) % 12;
        
        //日のと時の干支を求める
        int jd = ijday( year, month, day);
        if ( hour >= 23 && ! isLenient ) jd++;   //23時以降は次の日として計算
        //日の干支は節入時間の前後に影響されないのでシリアル日から直接求める。
        result.日干   = jd % 10;
        result.日支   = ( jd + 2 ) % 12;
        result.時支   = ( hour + 1 ) / 2 % 12;
        result.時干   = (((result.日干 % 5) * 12) + result.時支) % 10;
        
        //年家九星を求める
        int offset = (yk - 64) % 180;
        result.年三元 = offset / 60 % 3;
        result.年家九星 = 9 - ((yk + 7 ) % 9);

        //月家九星を求める
        int 月三元 = (y - 1) % 3;
        int 九星 = 10 - 月三元 * 3 - m;
        if ( 九星 == 0 ) 九星 = 9;
        else if ( 九星 < 0 ) 九星 += 9;
        result.月家九星 = 九星;
        result.月三元 = 月三元;
        
        //日家九星を求める
        get日家九星( jd, result);
        
        //時家九星を求める
        九星 = 0;
        int 時三元 = result.日支 % 3;
        offset = (時三元 * 12 + result.時支);
        if ( result.遁行 >= 1 ) {
            九星 = ( offset % 9) + 1;
        } else {
            九星 = (( 35 - offset) % 9) + 1;
        }
        result.時家九星 = 九星;
        result.時三元   = 時三元;
    }
    /**
     * カレンダーで指定された年月日時の干支と九星を求める。
     * @param cal カレンダー
     * @param result 結果が書きこまれる
     * 節入り判定は厳密に行う。
     */
    public static void getValue( Calendar cal, Kansi result ) {
        getValue( cal, result, false );
    }
    /**
     *　カレンダーで指定された日時の干支と九星を求め、Kansiオブジェクトを作成
     * してそれに書き込んで返す。節入り判定は厳密に行う。
     */
    public static Kansi getValue( Calendar cal ) {
        Kansi k = new Kansi();
        getValue( cal, k, false );
        return k;
    }
    /**
     *　カレンダーで指定された日時の干支と九星を求め、Kansiオブジェクトを作成
     * してそれに書き込んで返す。節入り判定は行わない。
     */
    public static Kansi getLenientValue( Calendar cal ) {
        Kansi k = new Kansi();
        getValue( cal, k, true);
        return k;
    }
}
