/*
 * JulianDay.java
 *
 * Created on 2008/01/26, 8:17
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.kyuureki;

import java.util.Calendar;
import java.util.TimeZone;
import static java.util.Calendar.*;
//import to.tetramorph.starbase.util.JDay;
/**
 * ユリウス日とグレゴリオ暦の相互変換をする。
 * @author 大澤義鷹
 */
class JulianDay {
    
    /**
     *  JulianDay オブジェクトを作成する
     */
    private JulianDay() {
    }
    /**
     * グレゴリオ暦年月日からユリウス日を求めて返す。
     * 紀元前1年はyear=0を指定。紀元前2年はyear=-1を指定。GregorianCalendarは
     * 紀元前の年は1年からでERAで紀元前を判定するため、その値をこのメソッドに
     * 渡すときは負数方式の値に変換してから入れなければならない。
     * 時刻の指定はできず、UTCの正午つまり、xx.5の値が返る。
     * @param year 1582-10-15からそれより未来の年
     * @param month 1-12を指定。
     * @param day 1-31を指定。
     */
    public static double get(int year,int month,int day) {
        int m = month;
        int y = year;
        if ( month < 3 ) {
            y--;
            m += 12;
        }
        int b = 0;
        if ( (year * 10000 + month * 100 + day) > 15821004 ) {
            int a = y / 100;
            b = 2 - a + (a / 4);
        }
        int c = ( y > 0 ) ? (int)(365.25 * y) : (int)(365.25 * y -0.75);
        return  1720994.5 + (int)( 30.6001 * ( m + 1 ) ) + b + c + day;
    }
    /**
     * ユリウス日をグレゴリオ暦に変換する。年月日の月は1～12。
     * 
     * @param get ユリウス日
     * @param tz   タイムゾーン
     */    
    public static Calendar getCalendar( double jday, TimeZone tz ) {
        double JD0 = 2440587.5; //1970年1月1日 UTC
        long millis=(long)(( jday - JD0 ) * 24L * 3600L * 1000L );
        Calendar c = Calendar.getInstance(tz);
        c.setTimeInMillis(millis);
        return c;
    }
    /**
     * カレンダーのユリウス日を返す。
     * これは一般的なユリウス日の計算アルゴリズムを使わず、システムのシリアル
     * タイムの規約に従って求める。つまり1970年1月1日UTCを基点に0として、
     * ミリ秒の単位で未来にインクリメント、過去にデクリメントしていく時刻系を
     * ユリウス日に変換する。非常にシンプルにユリウス日を求める事ができる。<br>
     * スイスエフェメリスのユリウス日取得メソッドとBC5400～AD5400まで値を比較
     * したが、ほとんどの場合、同じ値を取得できる。<br>
     * （１）注意すべきことは、カレンダーはミリ秒の精度をもっている。<br>
     * Calendar.getInstance()で現在時刻を取得したとき、ミリ秒の桁にも値がはいって
     * いて、年月日時分秒までセットしてこのメソッドでユリウス日に変換したとき、
     * ミリ秒の桁の値もユリウス日に反映される。<br>
     * 普通のユリウス日変換は、たいがい秒の桁までしか引数を取らないから、その値と
     * 比べると、わずかにずれていると感じるかもしれない。しかしそれはミリ秒桁を
     * リセットしていないのが原因。<br>
     * （２）カレンダークラスは日本が1948年から1951年まで実施していた、
     * サマータイムを認識している。次の例ではきっかり1時間遅れている。
     * <pre>
     * AD 1948-05-02(日) 23:32:09
     *    millis = -683720871000
     *    sd = 2432674.1056597224
     *    md = 2432674.0639930554,  sd - md = 0.041666666977107525
     * </pre>
     * ところが、Calendarクラスには、それを判定するメソッドが用意されていないのと、
     * TimeZone.useDaylightTimeは、この古い夏時間を認識しない。暗黙のうちに、
     * サマータイムの期間は1時間ずらして日時が求まる仕組みになっている。
     * 今いちよくわかっていないのだが、これは天体位置計算では要注意である。<br>
     * （３）ミリ秒桁がゼロになっている場合、ほとんどの場合はSweDate.getJulDayと
     * 一致するが、一致しない場合のもう一つの例は次のようなもの。
     * <pre>
     * BC 4669-11-03(土) 06:53:34
     *    millis = -209451751586000
     *    sd = 16377.412199074075
     *    md = 16377.412199074073
     * </pre>
     * 末尾の桁でわずかに値が異なる。これは計算誤差によるものだが、この桁だと
     * ピコ秒単位での違いでしかなくまったく問題にならない。
     */
    public static double get(Calendar cal) {
        double JD0 = 2440587.5; //1970年1月1日 UTC
        long ms = cal.getTimeInMillis();
        long dayms = 86400000L;
        long day = ms / dayms;
        double hour = ms % dayms;
        double jd = day + JD0 + (hour / dayms);
        return jd;
    }
    
//    /**
//     * ユリウス日をグレゴリオ暦に変換してyyyymmdd形式のint型で返す。
//     * 
//     * @param get ユリウス日
//     * @param tz   タイムゾーン
//     */
//    public static int revjday( double jday,TimeZone tz) {
//        Calendar c = getCalendar(jday,tz);
//        return c.get(YEAR) * 10000 
//                + (c.get(MONTH)+1) * 100 + c.get(DAY_OF_MONTH);
//    }
//    /**
//     * yyyymmddの形式の日付データを年月日に分解してymdに書きこむ。
//     * [0]=年,[1]=月,[2]=日。
//     */
//    public static void revIntDate( int date, int [] ymd) {
//        ymd[0] = date / 10000;
//        ymd[1] = (date - ( ymd[0] * 10000 )) / 100;
//        ymd[2] = date % 100;        
//    }
//    /**
//     * 年月日をyyyymmdd形式のint型データに変換して返す。
//     */
//    public static int dateIntValue(int year,int month,int day) {
//        return year * 10000 + month * 100 + day;
//    }
    
}
