/*
 * FormatterFactory.java
 *
 * Created on 2006/06/02, 20:52
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */

package to.tetramorph.starbase.formatter;

import javax.swing.JFormattedTextField;

/**
 * JFormattedTextFieldに自前のフォーマッタをセットするためのラッパークラス。
 * NetBeansのGUIデザイナを使わないのであれば、このようなクラスは本来必要ない。
 * GUIデザイナは最初に必ず引数無しのコンストラクタでウィジェトのインスタンスを作成
 * し、後からメソッドによって必要なパラメターを設定することが前提になっている。
 * <br>
 * JFormattedTextField用に自前で用意した日付や時刻のフォーマッタがたくさんあり、
 * それらはJFormattedTextField.AbstractFormatterをextendsして作成されたものだ。
 * 普通ならそれらのフォーマッタをコンストラクタで指定してJFormattedTextFieldの
 * インスタンスを作ればよいのだが、GUIデザイナの制約から指定することができない。
 * <br>
 * ではメソッドによってフォーマッタを後から設定してやれば良いかというと、
 * JFormattedTextField#setFormatter()はprotectedなので、
 * そのままでは同.AbstractFormatterをこのメソッドでセットすることはできない。
 * 子クラスを作りsetFormatterをオーバーライドすればよさそうだが、
 * 実際にやってみるとsetFormatter呼び出しと共にスタックオーバーフローのエラーを
 * 出して設定することができない。
 * <br>
 * そこでJFormattedTextField#setFormatterFactory()を使ってセットする方法を
 * ためしてみたらうまくいった。
 * <br>
 * このクラスのコンストラクタに自前のフォーマッタを与えてインスタンスを作り、
 * それをJFormattedTextField#setFormatterFactory(..)を使ってセットする。
 * <pre>
 * 例
 * JFormattedTextField date;
 * date.setFormatterFactory(new FormatterFactory(new MyDateFormatter()));
 * </pre>
 * @author 大澤義鷹
 */
public class FormatterFactory extends JFormattedTextField.AbstractFormatterFactory {
  private JFormattedTextField.AbstractFormatter fmt;
  /**
   * 自前のFormatterを与えて、インスタンスを作成する。
   * @param fmt JFormattedTextField.AbstractFormatterをextendsして作成したフォーマッタ。
   */
  public FormatterFactory(JFormattedTextField.AbstractFormatter fmt) {
    this.fmt = fmt;
  }
  /**
   * JFormattedTextField.AbstractFormatterFactory#getFormatterの実装で
   * コンストラクタで与えられたAbstractFormatterのインスタンスを返す。
   * @param tf プログラマは意識する必要なし。
   * @return フォーマッタを返す。
   */
  public JFormattedTextField.AbstractFormatter getFormatter(JFormattedTextField tf) {
    return fmt;
  }
}
