/*
 * FontChooser.java
 *
 * Created on 2011/07/31, 9:14:22
 *
 * プレビュー用のテキストエリアにはPrefferedSizeを設定している。エリアからあふれる
 * ほど大きなフォントが初期値で指定されたとき、レイアウトが崩れてしまうため。
 * 崩れてしまうと、ダイアログは固定サイズだしフォントサイズを手で入力するしか治す
 * 方法がなくなる。LaFがWindowsやMetal以外を指定されたとき、どうなるか不明で、
 * おそらく微妙にレイアウトがずれる可能性があるが仕方がない。すごくわずらわしい手は
 * 無いわけでもないが。
 *
 * 編集禁止のテキストフィールドにフォーカス取得しないように設定して、
 * 残りのコンポーネント間をタブキーで順番にフォーカスを移動できるようにしている。
 * フォントサイズリストのnextFocusableComponentをacceptButtonに設定することで、
 * プレビューテキストエリアにフォーカスが移るのをスキップしている。
 * (これは非推奨のメソッドだが一番簡単なのでまぁよしとした)
 * テキストエリアにフォーカスが移ってしまうとキー操作で抜け出せなくなるから。
 * テキストエリアをフォーカス禁止にすればよいかと思うかもしれないが、それをすると、
 * たとえ編集許可のフラグがオンでも、フォーカスを受け取らないので編集できなくなる。
 *
 * マウスによるダブクリでの選択＋決定はつけなかった。あまり必要性を感じないため。
 *
 * JListはCtrlキーを併用して選択することで、無選択の状態を作りだせてしまう。これを
 * 抑止するオプションはない。このため話がややこしくなる。このプログラムの中では、
 * 無選択はあってはならないのにそれができてしまう。それを判定するためにisSelected
 * など、本来不要なはずのメソッドを用意したり、ところどころわかりにくい判定を行って
 * いる。
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */
package to.tetramorph.fontchooser;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GraphicsEnvironment;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.AbstractAction;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * 汎用的なフォント選択ダイアログ。フォントファミリー、スタイル、サイズの３つの要素を
 * 選択し、それに応じたFileオブジェクトを返す。サイズは整数のポイント数のみ使用でき、
 * 実数のサイズのものは未対応。
 * @author ohsawa
 */
public class FontChooser extends javax.swing.JDialog {
    // スタイルのリストにセットされる文字列
    // セットされたあと変更されることはない
    static final String [] STYLE_NAMES = {
        "PLEAN","BOLD","ITALIC","BOLD ITALIC"
    };

    // サイズのリストにセットされる文字列。これも後から変更されることはない。
    static final String [] SIZES = {
        "3","5","8","10","12","14","16","18","24","36","48"
    };

    // 選択されたフォントがセットされる変数

    static Font chooseFont = null;

    /**
     * 使用可能なフォントファミリー名一覧。
     */
    public static final String [] FAMILY_NAMES;

    // 親がフレーム用のコンストラクタ

    FontChooser(java.awt.Frame parent) {
        super(parent, true);
        initComponents();
        init();
    }

    // 親がダイアログのコンストラクタ

    FontChooser(java.awt.Dialog parent) {
        super(parent, true);
        initComponents();
        init();
    }

    // 初期化時にフォントファミリー名の一覧表を作る

    static {
        // 使用可能なフォントファミリー一覧を作成
        GraphicsEnvironment genv = GraphicsEnvironment
                .getLocalGraphicsEnvironment();
        FAMILY_NAMES = genv.getAvailableFontFamilyNames();
//        Font[]   fonts  = genv.getAllFonts();
//        List<String> nameList = new ArrayList<String>();
//        for ( Font f : fonts ) {
//            if ( ! nameList.contains(f.getFamily()) ) {
//                nameList.add(f.getFamily());
//            }
//        }
//        Collections.sort(nameList);
//        FAMILY_NAMES = nameList.toArray();
    }
    /**
     * 指定されたファミリー名がJava環境の中に存在するならtrueを返す。
     * @param name フォントファミリー名
     */
    public static boolean isFamilyExists( String name ) {
        for ( int i = 0; i < FAMILY_NAMES.length; i++ ) {
            if ( FAMILY_NAMES[i].equalsIgnoreCase(name) ) return true;
        }
        return false;
    }
    // 初期化
    // リストに初期値やハンドラを登録したり、ダイアログのクローズイベント等を設定
    final void init() {
        // 各JListに選択項目をセット
        familyJList.setListData( FAMILY_NAMES );
        styleJList.setListData( STYLE_NAMES );
        sizeJList.setListData( SIZES );

        // 各JListに選択イベントハンドラをセット
        ListHandler lh = new ListHandler();
        familyJList.addListSelectionListener(lh);
        styleJList.addListSelectionListener(lh);
        sizeJList.addListSelectionListener(lh);

        //selectFontParams( getFont() );
        // サイズテキストフィールドにイベントハンドラをセット
        sizeTextField.addFocusListener(new SizeSelectHandler());

        // ESCキーでダイアログを閉じる設定
        ParentWindow.setEscCloseOperation(this, new AbstractAction() {
            @Override
            public void actionPerformed(ActionEvent evt) {
                cancelButton.doClick();
            }
        });

        // ダイアログの×ボタンで閉じたときの設定
        this.addWindowListener( new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                cancelButton.doClick();
            }
        });

        // プレビューテキストエリアのカーソルを先頭に移動
        prevTextArea.setCaretPosition(0);
    }
    /**
     * 指定されたフォントのパラメター通りに、
     * ダイアログ内の「ファミリー、スタイル、サイズ」のリストを選択状態にする。
     * @param font
     */
    void selectFontParams( Font font ) {
        boolean found = false;
        for ( int i = 0; i < FAMILY_NAMES.length; i++ ) {
            if ( FAMILY_NAMES[i].equals( font.getFamily() ) ) {
                familyJList.setSelectedIndex(i);
                familyJList.ensureIndexIsVisible(i);
                found = true;
            }
        }
        if ( ! found ) throw new IllegalArgumentException
                ("未知のフォントファミリー");

        int style = (font.isBold() ? 1 : 0 ) + (font.isItalic() ? 2 : 0);
        styleJList.setSelectedIndex( style );

        String sz = "" + font.getSize();

        found = false;
        for ( int i = 0; i < SIZES.length; i++ ) {
            if ( SIZES[i].equals(sz)) {
                sizeJList.setSelectedIndex(i);
                sizeJList.ensureIndexIsVisible(i);
                found = true;
                break;
            }
        }
        sizeTextField.setText(sz);
        if ( ! found )
            sizeJList.clearSelection();
        refrectParams();
    }

    // リストの選択状態を、リストの上に配置されているテキストフィールドに反映させる
    // サイズを反映させるのは、ちょっと細工している

    void refrectParams() {
        Object v = familyJList.getSelectedValue();
        Object empty = "";
        familyTextField.setText( (String)( v == null ? empty : v ) );
        v = styleJList.getSelectedValue();
        styleTextField.setText( (String)( v == null ? empty : v ) );

        v = sizeJList.getSelectedValue();
        String text = sizeTextField.getText();
        text = ( text == null ) ? "":text;
        if ( v == null || ! v.equals(text) ) {
            v = text;
        }
        sizeTextField.setText( (String)v );
    }

    // 各リストを検査して、フォントが選択されていると見なせるかどうかを判定する
    // 選択されているとみなせるときはtrueを返す。

    boolean isSelected() {
        if ( familyJList.getSelectedIndex() < 0 ) return false;
        if ( styleJList.getSelectedIndex() < 0 ) return false;
        if ( sizeJList.getSelectedIndex() < 0 ) {
            try {
                String num = sizeTextField.getText().trim();
                Integer.parseInt(num);
            } catch ( NumberFormatException e) {
                return false;
            }
        }
        return true;
    }

    // フォント選択処理
    // リストやテキストフィールドの値を元にフォントを作り変数chooseFontにセットする

    void selectFont() {
        String family = (String)familyJList.getSelectedValue();

        // Font.PLEAN=0, BOLD = 1, ITARIC = 2 BOLD+ITARIC=3となっているので、
        // リストの並び順をそのままFontのコンストラクタ引数に与えることができる

        int style = styleJList.getSelectedIndex();

        int size = 12; //異常な選択が行われときに採用するデフォルト値
        if ( sizeJList.getSelectedIndex() >= 0 ) {
            size = Integer.parseInt((String)sizeJList.getSelectedValue());
            sizeTextField.setText((String)sizeJList.getSelectedValue());
        } else {
            // サイズリストが無選択のときはサイズテキストフィールドの値を解釈する
            // 解釈できないときは先述の値12を採用する
            try {
                size = Integer.parseInt((sizeTextField.getText().trim()));
            } catch ( NumberFormatException ex) {
            }
        }
        Font f = new Font( family, style, size );
        prevTextArea.setFont(f);
        chooseFont = f;
        refrectParams();
        // 上記の処理でもすりぬける無選択状態はありうるので、最後に判定し無選択なら
        // 決定ボタンを押せなくする
        acceptButton.setEnabled(isSelected());
    }

    // サイズ入力テキストフィールドに値が入力され確定した（フォーカスが去った)ときに
    // フォント選択の処理を行う。

    class SizeSelectHandler implements FocusListener {

        @Override
        public void focusLost(FocusEvent e) {
            sizeJList.clearSelection();
            String num = sizeTextField.getText().trim();
            sizeJList.setSelectedValue( num, true );
            selectFont();
        }

        @Override
        public void focusGained(FocusEvent e) { }

    }

    // リストが選択されたときに呼び出され、フォント選択処理を行う

    class ListHandler implements ListSelectionListener {

        @Override
        public void valueChanged( ListSelectionEvent e ) {
            if ( e.getValueIsAdjusting() ) return;
            selectFont();
        }
    }

    /**
     * フォント選択ダイアログを開いて、選択をユーザに促し選択結果を返す。選択が中止さ
     * れた場合はnullを返す。
     * 必ずEDTから呼び出さなければならない。スレッドセーフなわけもない。
     *
     * @param parent 呼び出し元のComponent。
     * @param defFont 最初に選択状態にするフォント。nullを指定すると、このダイアログ
     * のデフォルトフォントが、選択された状態になって表示される。
     * @return 選択されたフォント。選択されなかったときはnullを返す。決定ボタンが
     * 押されたときは必ずフォントが戻る。中止ボタンやダイアログの×ボタンが押された
     * ときはかならずnullが戻る。
     */
    public static Font showDialog(Component parent, Font defFont) {
        Window window = ParentWindow.getWindowForComponent(parent);
        //SwingUtilities.getWindowAncestor(parent);
        final FontChooser dialog;
        if (window instanceof Frame) {
            dialog = new FontChooser((Frame) window);
        } else {
            dialog = new FontChooser((Dialog) window);
        }
        dialog.pack();
        if ( defFont == null ) defFont = dialog.getFont();
        dialog.selectFontParams(defFont);
        dialog.setComponentOrientation(window.getComponentOrientation());
        //親コンポーネントに対してセンタリング
        dialog.setLocationRelativeTo(window);
        dialog.setVisible(true);

        return chooseFont;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        javax.swing.JPanel jPanel5 = new javax.swing.JPanel();
        javax.swing.JPanel jPanel1 = new javax.swing.JPanel();
        javax.swing.JPanel jPanel2 = new javax.swing.JPanel();
        javax.swing.JScrollPane jScrollPane4 = new javax.swing.JScrollPane();
        prevTextArea = new javax.swing.JTextArea();
        javax.swing.JPanel jPanel3 = new javax.swing.JPanel();
        javax.swing.JPanel familyPanel = new javax.swing.JPanel();
        familyLabel = new javax.swing.JLabel();
        javax.swing.JScrollPane jScrollPane1 = new javax.swing.JScrollPane();
        familyJList = new javax.swing.JList();
        familyTextField = new javax.swing.JTextField();
        javax.swing.JPanel stylePanel = new javax.swing.JPanel();
        styleLabel = new javax.swing.JLabel();
        javax.swing.JScrollPane jScrollPane2 = new javax.swing.JScrollPane();
        styleJList = new javax.swing.JList();
        styleTextField = new javax.swing.JTextField();
        javax.swing.JPanel sizePanel = new javax.swing.JPanel();
        sizeLabel = new javax.swing.JLabel();
        javax.swing.JScrollPane jScrollPane3 = new javax.swing.JScrollPane();
        sizeJList = new javax.swing.JList();
        sizeTextField = new javax.swing.JTextField();
        javax.swing.JPanel jPanel4 = new javax.swing.JPanel();
        acceptButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("フォントの選択");
        setResizable(false);
        getContentPane().setLayout(new java.awt.GridLayout(1, 0));

        jPanel5.setBorder(javax.swing.BorderFactory.createEmptyBorder(10, 20, 1, 20));
        jPanel5.setLayout(new java.awt.BorderLayout());

        jPanel1.setLayout(new java.awt.GridBagLayout());

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("プレビュー"));
        jPanel2.setLayout(new java.awt.GridLayout(1, 0));

        jScrollPane4.setPreferredSize(new java.awt.Dimension(320, 113));

        prevTextArea.setColumns(20);
        prevTextArea.setRows(5);
        prevTextArea.setText("ここに文字を入力し見え方の確認ができます。\nABCDEF       123456789　　　　あいうえお\nabcdef       123456789　　　　ｱｲｳｴｵ");
        jScrollPane4.setViewportView(prevTextArea);

        jPanel2.add(jScrollPane4);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.PAGE_END;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 0);
        jPanel1.add(jPanel2, gridBagConstraints);

        jPanel3.setLayout(new java.awt.GridBagLayout());

        familyPanel.setLayout(new java.awt.GridBagLayout());

        familyLabel.setDisplayedMnemonic('F');
        familyLabel.setLabelFor(familyJList);
        familyLabel.setText("フォントファミリー(F)");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 4, 0);
        familyPanel.add(familyLabel, gridBagConstraints);

        jScrollPane1.setPreferredSize(new java.awt.Dimension(160, 140));

        familyJList.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        familyJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jScrollPane1.setViewportView(familyJList);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        familyPanel.add(jScrollPane1, gridBagConstraints);

        familyTextField.setEditable(false);
        familyTextField.setText("フォント名");
        familyTextField.setFocusable(false);
        familyTextField.setMargin(new java.awt.Insets(4, 4, 4, 4));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 4, 0);
        familyPanel.add(familyTextField, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 8);
        jPanel3.add(familyPanel, gridBagConstraints);

        stylePanel.setLayout(new java.awt.GridBagLayout());

        styleLabel.setDisplayedMnemonic('T');
        styleLabel.setLabelFor(styleJList);
        styleLabel.setText("スタイル(T)");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 4, 0);
        stylePanel.add(styleLabel, gridBagConstraints);

        jScrollPane2.setPreferredSize(new java.awt.Dimension(100, 140));

        styleJList.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        styleJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jScrollPane2.setViewportView(styleJList);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        stylePanel.add(jScrollPane2, gridBagConstraints);

        styleTextField.setEditable(false);
        styleTextField.setText("スタイル");
        styleTextField.setFocusable(false);
        styleTextField.setMargin(new java.awt.Insets(4, 4, 4, 4));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 4, 0);
        stylePanel.add(styleTextField, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 8);
        jPanel3.add(stylePanel, gridBagConstraints);

        sizePanel.setLayout(new java.awt.GridBagLayout());

        sizeLabel.setDisplayedMnemonic('S');
        sizeLabel.setLabelFor(sizeJList);
        sizeLabel.setText("サイズ(S)");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 4, 0);
        sizePanel.add(sizeLabel, gridBagConstraints);

        jScrollPane3.setPreferredSize(new java.awt.Dimension(50, 140));

        sizeJList.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        sizeJList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        sizeJList.setNextFocusableComponent(acceptButton);
        jScrollPane3.setViewportView(sizeJList);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        sizePanel.add(jScrollPane3, gridBagConstraints);

        sizeTextField.setText("12");
        sizeTextField.setMargin(new java.awt.Insets(4, 4, 4, 4));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 4, 0);
        sizePanel.add(sizeTextField, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        jPanel3.add(sizePanel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanel1.add(jPanel3, gridBagConstraints);

        jPanel5.add(jPanel1, java.awt.BorderLayout.CENTER);

        acceptButton.setMnemonic('Y');
        acceptButton.setText("決定(Y)");
        acceptButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                acceptButtonActionPerformed(evt);
            }
        });
        jPanel4.add(acceptButton);

        cancelButton.setMnemonic('N');
        cancelButton.setText("中止(N)");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });
        jPanel4.add(cancelButton);

        jPanel5.add(jPanel4, java.awt.BorderLayout.SOUTH);

        getContentPane().add(jPanel5);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        chooseFont = null;
        dispose();
    }//GEN-LAST:event_cancelButtonActionPerformed

    private void acceptButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_acceptButtonActionPerformed
        dispose();
    }//GEN-LAST:event_acceptButtonActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton acceptButton;
    private javax.swing.JButton cancelButton;
    private javax.swing.JList familyJList;
    private javax.swing.JLabel familyLabel;
    private javax.swing.JTextField familyTextField;
    private javax.swing.JTextArea prevTextArea;
    private javax.swing.JList sizeJList;
    private javax.swing.JLabel sizeLabel;
    private javax.swing.JTextField sizeTextField;
    private javax.swing.JList styleJList;
    private javax.swing.JLabel styleLabel;
    private javax.swing.JTextField styleTextField;
    // End of variables declaration//GEN-END:variables
}
