/*
 *
 * ----------
 * 
 * Copyright (C) 2007-2011 Yoshitaka Ohsawa All rights reserved.
 * This is free software with ABSOLUTELY NO WARRANTY.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * ----------
 */
package to.tetramorph.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
/**
 * 設定プロパティファイルなどのリソースを取得して返すメソッド群からなる。
 * 2011-07-28 ロガーを使うように修正。
 * @author 大澤義孝
 */
public class FileTools {
//    private static Font astroFont = null;
    //このｸﾗｽはｲﾝｽﾀﾝｽを作らせない
    private FileTools() {
    }

    /**
     * fileで指定されたプロパティファイルからデータをpropertiesにロードする。
     * 成功したときはtrueを返し、失敗(ファイルが存在しなかったり、フォーマットが
     * 異常だった場合)したときはfalseを返す。properitesにnullを渡すと戻り値はfalse。
     * @param properties データをロードしたいProperitesオブジェクト
     * @param file ロードするファイルオブジェクト。
     * @return ロードに成功すればtrue,失敗するとfalse。
     */
    public static boolean loadProperties( Properties properties, File file) {
        if(properties == null || file == null) return false;
        boolean no_err = true;
        if ( file.exists() ) {
            FileInputStream inputStream = null;
            try {
                inputStream = new FileInputStream(file);
                properties.loadFromXML(inputStream);
            } catch ( IOException e ) {
                Logger.getLogger(FileTools.class.getName())
                        .log(Level.SEVERE,null,e);
                no_err = false;
            } finally {
                try { inputStream.close(); } catch( Exception e ) { }
            }
        } else no_err = false;
        return no_err;
    }

    /**
     * inputStreamからXML形式のプロパティファイルをロードする。
     * ストリームから取得されたるプロパティはUTF-8で記述されていること。
     * @param properties このプロパティにデータがロードされる。
     * @param inputStream プロパティの入力ストリーム
     * @return ロードに成功したときはtrue,失敗するとfalse。
     */
    public static boolean loadProperties( Properties properties,
                                             InputStream inputStream) {
        if ( properties == null || inputStream == null ) return false;
        boolean no_err = true;
        try {
            properties.loadFromXML(inputStream);
        } catch ( IOException e ) {
            Logger.getLogger(FileTools.class.getName())
                    .log(Level.SEVERE,null,e);
            no_err = false;
        } finally {
            try { inputStream.close(); } catch(Exception e) { }
        }
        return no_err;
    }

    /**
     * 指定されたpropertiesの内容を指定されたfileに、XMLフォーマット(UTF-8)で
     * セーブする。書き込みに成功した場合はtrueを返す。
     * @param properties セーブしたいProperitesオブジェクト
     * @param file 書き出したいファイルオブジェクト。
     *             ファイル名は"*.properties"とするのが標準的。
     * @param comment プロパティファイル内に書かれる一行コメント。適当なコメント
     *                文字列であればなんでもよい。
     * @return セーブに成功したときはtrue。失敗したらfalse。
     */
    public static boolean saveProperties( Properties properties,
                                             File file,String comment ) {
        if ( properties == null ) return false;
        if ( file == null ) return false;
        if ( comment == null ) comment = "";
        boolean no_err = true;
        FileOutputStream outputStream = null;
        try {
            outputStream = new FileOutputStream(file);
            properties.storeToXML(outputStream,comment);
        } catch ( IOException e ) {
            Logger.getLogger(FileTools.class.getName())
                    .log(Level.SEVERE,null,e);
            no_err = false;
        } finally {
            try { outputStream.close(); } catch ( Exception e ) { }
        }
        return no_err;
    }
    /**
     * 入力ストリームからテキストを読みこむ。
     * 主にjar書庫のリソースからファイルを読み出すのに使用する。
     * 処理終了後、streamはクローズされる。
     * <pre>
     * InputStream is = Hoge.class.getResourceAsStream("/resources/hoge.txt");
     * StringBuilder sb = FileTools.loadText(is,"sjis");
     * </pre>
     * @param stream 入力ストリーム
     * @param charset 文字コードセット
     * @return 文字列が格納されたStringBuilder。IOExceptionで読みこみに失敗
     * したときはnullを返す。
     */
    public static StringBuilder loadText( InputStream stream,
                                                  String charset ) {
        StringBuilder sb = new StringBuilder(8192);
        BufferedReader br = null;
        try {
            InputStreamReader isr =  new InputStreamReader( stream, charset );
            br = new BufferedReader(isr);
            char [] buf = new char[4096];
            while( br.ready() ) {
                int size = br.read( buf, 0, 4096 );
                sb.append(buf, 0, size);
            }
            br.close();
        } catch ( IOException e ) {
            Logger.getLogger(FileTools.class.getName())
                    .log(Level.SEVERE,null,e);
            return null;
        } finally {
            try { br.close(); } catch ( Exception e ) { }
        }
        return sb;
    }
}
